#include <types.h>
#include <libsmd/vdp.h>
#include <libsmd/joy.h>
#include <libsmd/font.h>

u32	vtimer = 0;
u32	htimer = 0;

void
vdp_init(void)
{
  vdp_setreg(0, REG0_BASE);
  vdp_setreg(1, REG1_BASE | REG1_DMA);
  vdp_setreg(2, REG2_PLANEA(0xC000));
  vdp_setreg(3, REG3_WINDOW(0xB000));
  vdp_setreg(4, REG4_PLANEB(0xE000));
  vdp_setreg(5, REG5_SPRITE(0xBC00));
  vdp_setreg(7, REG7_PAL(0) | REG7_COLOR(0));
  vdp_setreg(10, REG_NULL);
  vdp_setreg(11, REG_NULL);
  vdp_setreg(12, REG_NULL);
  vdp_setreg(13, REG13_HSCROLL(0xB800));
  vdp_setreg(15, REG15_AUTOINC(2));	/* AutoInc 2 */
  vdp_setreg(16, REG_NULL);
  vdp_clrvram();
  vdp_clrvsram();
}

void
vblank(void)
{
}

void
hblank(void)
{
}

void
puts(u8 x, u8 y, char *str)
{
  vdp_pointto(0xC000 + (((x * 32) + y) * 2), VRAM_WRITE);
  while (*str)
    {
      vdp_write(*str);
      str++;
    }
}

void
putc(u8 x, u8 y, u8 c)
{
  vdp_pointto(0xC000 + (((x * 32) + y) * 2), VRAM_WRITE);
  vdp_write(c);
}

u8	do_joyinfo(u8 x, u8 y, u8 joy_num)
{
  u8	joy_type;

  joy_type = joy_gettype(joy_num);
  puts(x, y, "PAD ");
  putc(x, y + 4, '0' + joy_num + 1);
  puts(x + 2, y, "Type:");
  puts(x + 3, y, "Start:");
  puts(x + 4, y, "A:");
  puts(x + 5, y, "B:");
  puts(x + 6, y, "C:");
  puts(x + 7, y, "D-pad:");
  switch (joy_type)
    {
    case JOY_3BTS:
      puts(x + 2, y + 6, "3bts");
      break;
    case JOY_6BTS:
      puts(x + 2, y + 6, "6bts");
      puts(x + 8, y, "Mode:");
      puts(x + 9, y, "X:");
      puts(x + 10, y, "Y:");
      puts(x + 11, y, "Z:");
      break;
    case JOY_NONE:
      puts(x + 2, y + 6, "None");
      break;
    default:
      puts(x + 2, y + 6, "??  ");
      break;
    }
  return joy_type;
}

void
do_joy(u8 x, u8 y, u8 joy_num, u8 joy_type)
{
  u16	joy;

  joy = joy_read(joy_num, joy_type);
  putc(x, y, '0' + JOY_IS_PRESSED(joy, JOY_START));
  putc(x + 1, y, '0' + JOY_IS_PRESSED(joy, JOY_A));
  putc(x + 2, y, '0' + JOY_IS_PRESSED(joy, JOY_B));
  putc(x + 3, y, '0' + JOY_IS_PRESSED(joy, JOY_C));

  if (JOY_IS_PRESSED(joy, JOY_UP))
    {
      if (JOY_IS_PRESSED(joy, JOY_RIGHT))
	puts(x + 4, y, "UR");
      else if (JOY_IS_PRESSED(joy, JOY_LEFT))
	puts(x + 4, y, "UL");
      else
	puts(x + 4, y, "U_");
    }
  else if (JOY_IS_PRESSED(joy, JOY_DOWN))
    {
      if (JOY_IS_PRESSED(joy, JOY_RIGHT))
	puts(x + 4, y, "DR");
      else if (JOY_IS_PRESSED(joy, JOY_LEFT))
	puts(x + 4, y, "DL");
      else
	puts(x + 4, y, "D_");
    }
  else if (JOY_IS_PRESSED(joy, JOY_LEFT))
    puts(x + 4, y, "_L");
  else if (joy & JOY_RIGHT)
    puts(x + 4, y, "_R");
  else
    puts(x + 4, y, "__");
  if (joy_type == JOY_6BTS)
    {
      putc(x + 5, y, '0' + JOY_IS_PRESSED(joy, JOY_MODE));
      putc(x + 6, y, '0' + JOY_IS_PRESSED(joy, JOY_X));
      putc(x + 7, y, '0' + JOY_IS_PRESSED(joy, JOY_Y));
      putc(x + 8, y, '0' + JOY_IS_PRESSED(joy, JOY_Z));
    }
}

void
main(void)
{
  u8	joy_type[4];
  u8	ea4way = 0;

  vdp_init();
  vdp_dma(32, font_pal1, 0, DMA_TO_CRAM);
  vdp_waitdma();
  vdp_dma(sizeof(font_ascii), font_ascii, 0, DMA_TO_VRAM);
  vdp_waitdma();
  vdp_setreg(1, REG1_BASE | REG1_DISPLAY);

  joy_init();

  if (joy_isea4way())
    {
      puts(0, 7, "EA4WAY Detected");
      ea4way = 1;
      joy_ea4way_set(JOY_1);
      joy_type[JOY_1] = do_joyinfo(1, 1, JOY_1);
      joy_ea4way_set(JOY_2);
      joy_type[JOY_2] = do_joyinfo(1, 16, JOY_1);
      joy_ea4way_set(JOY_3);
      joy_type[JOY_3] = do_joyinfo(16, 1, JOY_1);
      joy_ea4way_set(JOY_4);
      joy_type[JOY_4] = do_joyinfo(16, 16, JOY_1);
    }
  else
    {
      joy_type[JOY_1] = do_joyinfo(1, 1, JOY_1);
      joy_type[JOY_2] = do_joyinfo(1, 16, JOY_2);
    }

  while (1)
    {
      /* Proceed joy 1 */
      if (ea4way)
	joy_ea4way_set(JOY_1);
      switch (joy_type[JOY_1])
	{
	case JOY_3BTS:
	  do_joy(4, 8, JOY_1, JOY_3BTS);
	  break;
	case JOY_6BTS:
	  do_joy(4, 8, JOY_1, JOY_6BTS);
	  break;
	}

      /* Proceed joy 2 */
      if (ea4way)
	joy_ea4way_set(JOY_2);
      switch (joy_type[JOY_2])
	{
	case JOY_3BTS:
	  if (ea4way)
	    do_joy(4, 23, JOY_1, JOY_3BTS);
	  else
	    do_joy(4, 23, JOY_2, JOY_3BTS);
	  break;
	case JOY_6BTS:
	  if (ea4way)
	    do_joy(4, 23, JOY_1, JOY_6BTS);
	  else
	    do_joy(4, 23, JOY_2, JOY_6BTS);
	  break;
	}

      /* Do joy 3/4 is ea4way is plugged */

      if (ea4way)
	{
	  joy_ea4way_set(JOY_3);
	  switch (joy_type[JOY_3])
	    {
	    case JOY_3BTS:
	      do_joy(19, 8, JOY_1, JOY_3BTS);
	      break;
	    }
	  joy_ea4way_set(JOY_4);
	  switch (joy_type[JOY_4])
	    {
	    case JOY_3BTS:
	      do_joy(19, 23, JOY_1, JOY_3BTS);
	      break;
	    }
	}
      vdp_waitvsync();
    }
}
