#include <types.h>
#include <libsmd/vdp.h>
#include <libsmd/joy.h>

#include "backt.h"
#include "backm.h"
#include "backp.h"

#include "chart.h"
#include "charp.h"

u16	vtimer = 0;
u16	htimer = 0;

s16	hscroll = 0;
s16	vscroll = 0;

void
vdp_init(void)
{
  vdp_setreg(0, REG0_BASE);
  vdp_setreg(1, REG1_BASE | REG1_DMA);
  vdp_setreg(2, REG2_PLANEA(0xC000));
  vdp_setreg(3, REG3_WINDOW(0xB000));
  vdp_setreg(4, REG4_PLANEB(0xE000));
  vdp_setreg(5, REG5_SPRITE(0xBC00));
  vdp_setreg(7, REG7_PAL(0) | REG7_COLOR(0));
  vdp_setreg(10, REG_NULL);
  vdp_setreg(11, REG11_HFULL | REG11_VFULL);
  vdp_setreg(12, REG_NULL);
  vdp_setreg(13, REG13_HSCROLL(0xB800));
  vdp_setreg(15, REG15_AUTOINC(2));
  vdp_setreg(16, REG16_64X64);
  vdp_clrvram();
  vdp_clrvsram();
}

void
vblank(void)
{
}

void
hblank(void)
{
}

void
main(void)
{
  u8	joy;
  u32	sprnum;
  u32	sprbase;
  u32	posx, posy;

  vdp_init();
  vdp_writetocram(32, back_pal, 0);
  vdp_writetocram(32, char_pal, 32);
  vdp_dma(sizeof(back_tiles), back_tiles, 0, DMA_TO_VRAM);
  vdp_waitdma();
  vdp_dma(sizeof(char_tiles), char_tiles, sizeof(back_tiles), DMA_TO_VRAM);
  vdp_waitdma();

  vdp_writetovram(sizeof(back_map), back_map, 0xC000);

  vdp_setreg(1, REG1_BASE | REG1_VINT | REG1_DISPLAY);

  sprbase = sizeof(back_tiles)/8/4;
  sprnum = sprbase;
  posx = 240;
  posy = 240;

  joy_init();
  while (1)
    {
      joy = joy_read(JOY_1, JOY_3BTS);
      switch (joy)
	{
	case JOY_RIGHT:
	  sprnum = sprbase + (9 * 3 * 4);
	  if (hscroll > -256)
	    hscroll -= 3;
	  else if (posy < 348)
	    posy += 2;
	  break;
	case JOY_LEFT:
	  sprnum = sprbase + (6 * 3 * 4);
	  if (hscroll < 0)
	    hscroll += 3;
	  else if (posy > 138)
	    posy -= 2;
	  break;
	case JOY_UP:
	  sprnum = sprbase + (3 * 3 * 4);
	  if (vscroll > 0)
	    vscroll -= 3;
	  else if (posx > 130)
	    posx -= 2;
	  break;
	case JOY_DOWN:
	  sprnum = sprbase;
	  if (vscroll <= 286)
	    vscroll += 3;
	  else if (posx < 270)
	    posx += 2;
	  break;
	}
      /* Write the hscroll data */
      vdp_pointto(0xB800, VRAM_WRITE);
      vdp_write(hscroll);

      /* Write the vscroll data */
      vdp_pointto(0, VSRAM_WRITE);
      vdp_write(vscroll);

      /* Drawing the sprite */
      vdp_pointto(0xBC00, VRAM_WRITE);
      vdp_write32((posx << 16) | (SPRITE_SIZE_24X32 | 1));
      vdp_write32((VDP_BUILD_TILE(sprnum, 0, 0, 1, 0) << 16) | posy);
      vdp_waitvsync();
    }
}
