/*
 * Copyright 2009-2010 The MegaDrive.Org Team
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *    1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MEGADRIVE.ORG TEAM ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EMMANUEL VADOT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* $Id: joy.s,v 1.5 2010/01/11 06:32:36 elbarto Exp $ */

.global joy_init
.global joy_gettype
.global joy_read
.global joy_getport
.global	joy_isea4way
.global joy_ea4way_set

JOY1_DATA = 0xa10003
JOY2_DATA = 0xa10005
EXT_DATA = 0xa10007

JOY1_CTRL = 0xa10009
JOY2_CTRL = 0xa1000b
EXT_CTRL = 0xa1000d

joy_init:
	move.b	#0x40, (JOY1_CTRL)
	move.b	#0x40, (JOY2_CTRL)
	move.b	#0x40, (EXT_CTRL)
	rts

joy_getport:
	move.l	4(%a7), %d0
	lea	JOY1_DATA, %a0
	mulu.w	#2, %d0
	add.l	%d0, %a0

	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d0
	move.b	#0x40, (%a0)
	nop
	lsl.w	#8, %d0
	move.b	(%a0), %d0
	rts

joy_gettype:
	move.l	4(%a7), %d0
	lea	JOY1_DATA, %a0
	mulu.w	#2, %d0
	add.l	%d0, %a0

* Test is something is attached to port
	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d0
	move.b	#0x40, (%a0)
	nop
	nop
	andi.b	#0xC, %d0
	bne.w	nopad

* Test 6 buttons pad
	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d0
	move.b	#0x40, (%a0)
	nop
	nop
	move.b	(%a0), %d0
	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d0
	move.b	#0x40, (%a0)
	nop
	nop
	andi.w	#0x33, %d0
	cmpi.w	#0x33, %d0
	bne	joy6bts

* Classic 3 buttons joypad
joy3bts:
	move.w	#0x1, %d0
	rts
joy6bts:
	move.w	#0x2, %d0
	rts
nopad:
	move.w	#0x0, %d0
	rts

joy_read:
	move.l	4(%a7), %d0
	lea	JOY1_DATA, %a0
	mulu.w	#2, %d0
	add.l	%d0, %a0
	bsr	joy_poll
	move.w	8(%a7), %d1
	cmpi.w	#1, %d1
	beq	read3bts

*We want to read a 6 buttons pad
	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d1
	move.b	#0x40, (%a0)
	nop
	nop
	move.b	(%a0), %d1
	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d1
	move.b	#0x40, (%a0)
	nop
	nop
	move.b	(%a0), %d1
	lsl.w	#8, %d1
	or.w	%d1, %d0
read3bts:
	not.w	%d0
	rts

* Routine to poll data from joystick, %a0 contain the port address to use, %d0 the result
joy_poll:
	move.b	#0x0, (%a0)
	nop
	nop
	move.b	(%a0), %d1
	andi.b	#0x30, %d1
	lsl.b	#2, %d1
	move.b	#0x40, (%a0)
	nop
	nop
	move.b	(%a0), %d0
	andi.b	#0x3F, %d0
	or.b	%d1, %d0
	rts

joy_isea4way:
* Test EA4WAY/SEGA Team Player V2
	move.b	#0x40, (JOY1_CTRL)
	nop
	move.b	#0x7F, (JOY2_CTRL)
	nop
	move.b	#0x7C, (JOY2_DATA)
	nop
	move.b	(JOY1_DATA), %d0
	andi.b	#0x3, %d0
	beq.w	ea4way
	move.w	#0, %d0
* Crappy hack
	move.b	#0x40, (JOY2_CTRL)
	move.w	#0x50, %d1
crappyhack:
	dbra	%d1, crappyhack
	rts
ea4way:
	move.w	#1, %d0
	rts


joy_ea4way_set:
	move.l	4(%a7), %d0
	lea	JOY2_DATA, %a0
	lsl.w	#4, %d0
	or.w	#0xC, %d0
	move.b	%d0, (JOY2_DATA)
	nop
	nop
	rts
