/*
 * Copyright 2009-2010 The MegaDrive.Org Team
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *    1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MEGADRIVE.ORG TEAM ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EMMANUEL VADOT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* $Id: vdp.s,v 1.5 2010/01/11 06:32:36 elbarto Exp $ */
	
.global vdp_setreg
.global	vdp_pointto
.global vdp_write
.global vdp_write16
.global vdp_write32
.global	vdp_writetocram
.global	vdp_writetovram
.global	vdp_writetovsram
.global vdp_dma
.global vdp_dmafill
.global	vdp_waitdma
.global vdp_clrvram
.global vdp_clrvsram
.global vdp_waitvsync

VDP_DATA =		0x00c00000
VDP_CTRL =		0x00c00004
VDP_HVCOUNTER =		0x00c00008

vdp_setreg:
	movem.l	4(%a7), %d0-%d1

	lsl.w	#8, %d0
	or.w	#0x8000, %d0
	or.w	%d1, %d0

	move.w	%d0, (VDP_CTRL)
	rts

vdp_pointto:
	movem.l	4(%a7), %d0-%d1

	and.w	#0x3FFF, %d0
	and.w	#3, %d1
	lsl.w	#8, %d1
	lsl.w	#6, %d1
	or.w	%d1, %d0
	move.w	%d0, (VDP_CTRL)

	movem.l	4(%a7), %d0-%d1
	lsr.w	#8, %d0
	lsr.w	#6, %d0
	and.w	#0x3C, %d1
	lsl.w	#2, %d1
	or.w	%d1, %d0
	move.w	%d0, (VDP_CTRL)

	rts

vdp_write:
vdp_write16:
	movem.l	4(%a7), %d0
	move.w	%d0, (VDP_DATA)
	rts

vdp_write32:
	movem.l	4(%a7), %d0
	move.l	%d0, (VDP_DATA)
	rts

vdp_writetocram:
	movem.l	4(%a7), %d1/%a0
	move.l	12(%a7), %d0

	or.w	#0xc000, %d0

	move.w	%d0, (VDP_CTRL)
	move.w	#0, (VDP_CTRL)

_writetocram:
	move.w	(%a0)+, (VDP_DATA)
	dbra	%d1, _writetocram
	rts

vdp_writetovram:
	/* Write destination address (1st cmd) */
	move.l	12(%a7), %d0
	and.w	#0x3FFF, %d0
	or.w	#0x4000, %d0
	move.w	%d0, (VDP_CTRL)

	/* 2nd cmd */
	move.l	12(%a7), %d0
	lsr.w	#8, %d0
	lsr.w	#6, %d0
	move.w	%d0, (VDP_CTRL)

	movem.l	4(%a7), %d0/%a0
	divu.w	#2, %d0
	subq.w	#1, %d0
_writetovram:
	move.w	(%a0)+, (VDP_DATA)
	dbra	%d0, _writetovram
	rts

vdp_writetovsram:
	/* Write destination address (1st cmd) */
	move.l	12(%a7), %d0
	and.w	#0x3FFF, %d0
	or.w	#0x4000, %d0
	move.w	%d0, (VDP_CTRL)

	/* 2nd cmd */
	move.l	12(%a7), %d0
	lsr.w	#8, %d0
	lsr.w	#6, %d0
	or.w	#0x10, %d0
	move.w	%d0, (VDP_CTRL)

	movem.l	4(%a7), %d0/%a0
	divu.w	#2, %d0
	subq.w	#1, %d0
_writetovsram:
	move.w	(%a0)+, (VDP_DATA)
	dbra	%d0, _writetovsram
	rts

vdp_dma:
	move.l	4(%a7), %d0
	/* Setup DMA length */
	lsr.w	#1, %d0
	and.w	#0xff, %d0
	or.w	#0x9300, %d0
	move.w	%d0, (VDP_CTRL)
	move.l	4(%a7), %d0
	lsr.w	#8, %d0
	lsr.w	#1, %d0
	and.w	#0xff, %d0
	or.w	#0x9400, %d0
	move.w	%d0, (VDP_CTRL)

	/* Setup DMA source address */
	/* Low */
	move.l	8(%a7), %d0
	lsr.l	#1, %d0
	and.l	#0xff, %d0
	or.l	#0x9500, %d0
	move.w	%d0, (VDP_CTRL)
	/* Mid */
	move.l	8(%a7), %d0
	lsr.l	#8, %d0
	lsr.l	#1, %d0
	and.l	#0xff, %d0
	or.l	#0x9600, %d0
	move.w	%d0, (VDP_CTRL)
	/* High */
	move.l	8(%a7), %d0
	lsr.l	#8, %d0
	lsr.l	#8, %d0
	lsr.l	#1, %d0
	and.l	#0xff, %d0
	or.l	#0x9700, %d0
	move.w	%d0, (VDP_CTRL)

	/* Setup DMA dest address */
	/* 1st cmd */
	move.l	12(%a7), %d0
	move.l	16(%a7), %d1
	and.w	#0x3FFF, %d0
	lsl.w	#8, %d1
	lsl.w	#6, %d1
	or.w	%d1, %d0
	move.w	%d0, (VDP_CTRL)
	/* 2nd cmd */
	move.l	12(%a7), %d0
	move.l	16(%a7), %d1
	lsr.w	#8, %d0
	lsr.w	#6, %d0
	and.w	#0x4, %d1
	lsl.w	#2, %d1
	or.w	#0x80, %d1
	or.w	%d1, %d0
	/* push to 2nd cmd to ram */
	move.w	%d0, -(%a7)
	move.w	(%a7), (VDP_CTRL)
	move.w	(%a7)+, %d0
	rts

vdp_dmafill:
	/* Set autoinc to 1 */
	move.w	#0x8F01, (VDP_CTRL)
	
	/* Setup DMA length */
	move.l	4(%a7), %d0
	and.w	#0xff, %d0
	or.w	#0x9300, %d0
	move.w	%d0, (VDP_CTRL)
	move.l	4(%a7), %d0
	and.w	#0xff00, %d0
	lsr.w	#8, %d0
	or.w	#0x9400, %d0
	move.w	%d0, (VDP_CTRL)

	/* Set DMA fill mode */
	move.w	#0x9780, (VDP_CTRL)
	
	/* Setup DMA dest address */
	/* 1st cmd */
	move.l	12(%a7), %d0
	and.w	#0x3FFF, %d0
	or.w	#0x4000, %d0
	move.w	%d0, (VDP_CTRL)
	/* 2nd cmd */
	move.l	12(%a7), %d0
	lsr.w	#8, %d0
	lsr.w	#6, %d0
	or.w	#0x80, %d0
	/* push to 2nd cmd to ram */
	move.w	%d0, -(%a7)
	move.w	(%a7), (VDP_CTRL)
	move.w	(%a7)+, %d0

	/* Write fill data to vdp data port */
	move.w	8(%a7), %d0
	move.w	%d0, (VDP_DATA)
	
	rts

vdp_waitdma:
	move.w	(VDP_CTRL), %d0
_waitdma:
	btst	#1, %d0
	bne.s	_waitdma
	rts

vdp_clrvram:
	move.l	#0x60000000, (VDP_CTRL)
	move.w	#0x0,%d0
_clrvram:
	move.w	#0, (VDP_DATA)
	subq.w	#2, %d0
	bne	_clrvram
	rts

vdp_clrvsram:
	move.l	#0x40000010, (VDP_CTRL)
	move.w	#80,%d0
_clrvsram:
	move.w	#0, (VDP_DATA)
	subq.w	#2, %d0
	bne	_clrvsram
	rts

vdp_waitvsync:
	move.w	(VDP_CTRL), %d0
	btst #3,%d0
	bne.s vdp_waitvsync
_waitvsync:
	move.w (VDP_CTRL),%d0
	btst #3,%d0
	beq.s _waitvsync

	rts
